from kivy.uix.widget import Widget
from kivy.graphics import *

import game_manager, util

IDLE, COLLIDING, FAILED = xrange(3)


class Goal(Widget):
    colliding_scale = 4

    radius = 2
    r_sqrd = radius ** 2
    colliding_r_sqrd = (radius * colliding_scale) ** 2

    rotation_speed = 50
    acceleration = 50

    shrink_speed = 3

    def __init__(self, app):
        self.app = app
        Widget.__init__(self)
        with self.canvas:
            PushMatrix()
            self.position = Translate()
            self.rotation = Rotate(0, 0, 1, 0)
            self.scale = Scale()

            self.app.graphics.draw_scene('models/goal')
            PopMatrix()

        self.state = IDLE

        self.min_speed = self.rotation_speed

    def update(self, dt):
        if self.state == FAILED:
            if self.scale.x > 0:
                s = self.scale.x - self.shrink_speed * dt
                if s < 0:
                    s = 0
                self.scale.x = s
                self.scale.z = s
        else:
            dist_sqrd = util.sqr_distance_between(self.position.xyz[::2],
                                                  self.app.game_manager.player.position.xyz[::2])
            if self.state == COLLIDING:
                self.rotation_speed += self.acceleration * dt

                if dist_sqrd > self.colliding_r_sqrd:
                    self.state = IDLE
                    self.app.audio_manager.play_sound('powerdown')
                    self.app.game_manager.set_state(game_manager.PLAYING)

                elif self.scale.x < self.colliding_scale:
                    s = self.scale.x * 1.01
                    if s > self.colliding_scale:
                        s = self.colliding_scale
                    self.scale.x = s
                    self.scale.z = s
            elif self.state == IDLE:
                if self.rotation_speed > self.min_speed:
                    self.rotation_speed -= self.acceleration * dt

                if dist_sqrd < self.r_sqrd:
                    self.state = COLLIDING
                    self.app.audio_manager.play_sound('powerup')
                    self.app.game_manager.set_state(game_manager.LEAVING)

                elif self.scale.x > 1:
                    s = self.scale.x * .99
                    if s < 1:
                        s = 1
                    self.scale.x = s
                    self.scale.z = s

        self.rotation.angle += self.rotation_speed * dt
