import os

from kivy.uix.image import Image
from kivy.graphics import *
from kivy.resources import resource_find

from objloader import ObjFile


class GraphicData:
    textures = {}
    scenes = {}

    def get_scene(self, filename):
        if filename in self.scenes:
            return self.scenes[filename]
        else:
            return self.load_scene(filename)

    def load_scene(self, filename):
        path = '{}.obj'.format(filename)
        if os.path.exists(path):
            scene = ObjFile(path)
            for material in scene.mtl.itervalues():
                if 'map_Kd' in material:
                    tex_filename = material['map_Kd'][0].split('\\\\')[-1]
                    material['tex'] = self.get_texture(tex_filename)
                else:
                    material['tex'] = None
            self.scenes[filename] = scene
            return scene
        else:
            print '{} does not exist'.format(path)

    def get_texture(self, filename):
        if filename in self.textures:
            return self.textures[filename]
        else:
            return self.load_texture(filename)

    def load_texture(self, filename):
        path = 'textures/{}'.format(filename)
        if os.path.exists(path):
            tex = Image(source=path).texture
            tex.wrap = 'repeat'
            self.textures[filename] = tex
            return tex
        else:
            print '{} does not exist'.format(path)

    def draw_scene(self, name):
        scene = self.get_scene(name)
        for obj in scene.objects.itervalues():
            m = Mesh(vertices=obj.vertices,
                     indices=obj.indices,
                     fmt=obj.vertex_format,
                     mode='triangles')
            if obj.material_name is not None:
                m.texture = scene.mtl[obj.material_name]['tex']