from kivy.core.window import Window
from kivy.uix.widget import Widget


class InputManager(Widget):

    def __init__(self, app):
        self.app = app
        Widget.__init__(self)

        self._keyboard = Window.request_keyboard(self._keyboard_closed, self, 'text')
        self._keyboard.bind(on_key_down=self._on_keyboard_down)
        self._keyboard.bind(on_key_up=self._on_keyboard_up)

        self.last_pressed = None

    def _keyboard_closed(self):
        self._keyboard.unbind(on_key_down=self._on_keyboard_down)
        self._keyboard.unbind(on_key_up=self._on_keyboard_up)
        self._keyboard = None

    def _on_keyboard_down(self, keyboard, keycode, text, modifiers):
        key_id, key_text = keycode

        if key_id != self.last_pressed:
            self.last_pressed = key_id

            if key_text == 'escape':
                self.app.on_esc()
            elif key_text == 'w':
                self.app.game_manager.player.on_axis(1, 1)
            elif key_text == 's':
                self.app.game_manager.player.on_axis(1, -1)
            elif key_text == 'd':
                self.app.game_manager.player.on_axis(0, 1)
            elif key_text == 'a':
                self.app.game_manager.player.on_axis(0, -1)
            elif key_text == 'spacebar':
                self.app.game_manager.player.attack()
            else:
                pass  # print keycode

        return True

    def _on_keyboard_up(self, keyboard, keycode):
        key_id, key_text = keycode

        if key_text == 'w':
            self.app.game_manager.player.on_axis(1, -1)
        elif key_text == 's':
            self.app.game_manager.player.on_axis(1, 1)
        elif key_text == 'd':
            self.app.game_manager.player.on_axis(0, -1)
        elif key_text == 'a':
            self.app.game_manager.player.on_axis(0, 1)
        elif key_text == 'spacebar':
            self.app.game_manager.player.stop_attack()

        if key_id == self.last_pressed:
            self.last_pressed = None

        return True
