from kivy.uix.widget import Widget
from kivy.graphics import *

import util

IDLE, MOVE, COLLECT = xrange(3)


class Item(Widget):
    r_sqrd = 25
    rotation_speed = 50

    collect_r_sqrd = 4

    def __init__(self, app, name):
        self.app = app
        self.name = name
        Widget.__init__(self)
        with self.canvas:
            PushMatrix()

            self.position = Translate()
            self.rotation = Rotate(0, 0, 1, 0)

            self.scale = Scale()

            Color(rgb=(1, 1, 1))
            self.app.graphics.draw_scene('models/cone')

            PopMatrix()

        self.state = IDLE
        self.state_timer = 0

    def set_state(self, state):
        if state == IDLE:
            self.scale.x = 1
            self.scale.y = 1
            self.scale.z = 1
        elif state == COLLECT:
            self.app.audio_manager.play_sound('click')
            self.state_timer = 1
        elif state == MOVE:
            self.state_timer = 0
        self.state = state

    def update(self, dt):
        self.rotation.angle += self.rotation_speed * dt

        if self.state is COLLECT:
            self.state_timer -= dt
            if self.state_timer <= 0:
                self.app.game_data.get_item(self.name)
                self.app.game_data.count('items')
                self.app.game_manager.remove_item(self)
            else:
                self.scale.x = self.state_timer
                self.scale.y = self.state_timer
                self.scale.z = self.state_timer

        else:
            dist_sqrd = util.sqr_distance_between(self.position.xyz[::2],
                                                  self.app.game_manager.player.position.xyz[::2])

            if self.state is MOVE:
                if dist_sqrd < self.collect_r_sqrd:
                    self.set_state(COLLECT)
                elif dist_sqrd > self.r_sqrd:
                    self.set_state(IDLE)
                else:
                    self.state_timer += dt
                    s = util.loop_time(self.state_timer, .5) * .3 + .7
                    self.scale.x = s
                    self.scale.z = s

            elif self.state is IDLE:
                if dist_sqrd < self.r_sqrd:
                    self.set_state(MOVE)
